/*
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include "base/Log.h"
#include "base/util/utils.h"

Log LOG = Log(false);

char logmsg[512];

static FILE* logFile = NULL;
static char logFullName[512] = "\\" LOG_NAME ;    
static char logName[128] = LOG_NAME;
static char logPath[256] = "\\" ;   

//---------------------------------------------------------------------- Static Functions

/*
* return a the time to write into log file. If complete is true, it return 
* the date too, else only hours, minutes, seconds and milliseconds
*/ 
static char*  getCurrentTime(BOOL complete) {
    
    SYSTEMTIME sys_time;   
    TIME_ZONE_INFORMATION timezone;
    GetLocalTime(&sys_time);
    GetTimeZoneInformation(&timezone);

    char fmtComplete[] = T("%04d-%02d-%02d %02d:%02d:%02d GMT %c%d:%02d");
    char fmt[]         = T("%02d:%02d:%02d GMT %c%d:%02d");

    char*  ret = new char [64];
    
    // calculate offset from UTC/GMT in hours:min, positive value means east of Greenwich (e.g. CET = GMT +1)
    
    char direction = timezone.Bias <= 0 ? '+' : '-';    
    int hours = abs(timezone.Bias / 60) ;
    int minutes = abs(timezone.Bias % 60);

    if (complete) {
        sprintf(ret, fmtComplete, sys_time.wYear, sys_time.wMonth, sys_time.wDay,
                 sys_time.wHour, sys_time.wMinute, sys_time.wSecond,
                direction, hours, minutes);
    } else {
        sprintf(ret, fmt, sys_time.wHour, sys_time.wMinute, sys_time.wSecond, 
                direction, hours, minutes);
    }
    return ret;
}

//---------------------------------------------------------------------- Constructors

Log::Log(BOOL resetLog, char*  path, char*  name) {

    setLogPath(path);
    setLogName(name);
	if (resetLog) {
        reset("Funambol WindowsMobile Plugin Log");
    }
}

Log::~Log() {
    if (logFile != NULL) {
        fclose(logFile);
    }
}

//---------------------------------------------------------------------- Public methods

void Log::setLogPath(char*  configLogPath) {
    
    if (configLogPath != NULL) {
        sprintf(logPath, T("%s/"), configLogPath); 
    } else {
        sprintf(logPath, T("%s"), T("./"));
    }
}

void Log::setLogName(char*  configLogName) {
    
    if (configLogName != NULL) {
        sprintf(logName, T("%s"), configLogName); 
    }
    else {
        sprintf(logName, T("%s"), LOG_NAME);         
    }
}

void Log::error(const char*  msg, ...) {    
    va_list argList;
    va_start (argList, msg);
    printMessage(LOG_ERROR, msg, argList);    
    va_end(argList);
}

void Log::info(const char*  msg, ...) {
    if (logLevel >= LOG_LEVEL_INFO) {
        va_list argList;
	    va_start (argList, msg);
        printMessage(LOG_INFO, msg, argList);    
	    va_end(argList);

    }
}

void Log::debug(const char*  msg, ...) {
    if (logLevel >= LOG_LEVEL_DEBUG) {
	    va_list argList;
        va_start (argList, msg);
        printMessage(LOG_DEBUG, msg, argList);    
        va_end(argList);
        
    }
}

void Log::trace(const char*  msg) { 
}

void Log::setLevel(LogLevel level) {
    logLevel = level;
}

LogLevel Log::getLevel() {
    return logLevel;
}

BOOL Log::isLoggable(LogLevel level) {
    return (level >= logLevel);
}

void Log::printMessage(const char*  level, const char*  msg, va_list argList) {       
    
	char*  currentTime = getCurrentTime(false);    
    logFile = fopen(logFullName, "a+");       
	
	fprintf(logFile, "%s [%s] - ", currentTime, level); 		
    vfprintf(logFile, msg, argList);	
	fprintf(logFile, "\n"); 
	fclose(logFile);

    delete[] currentTime;	
}

void Log::printMessageW(const char* level, const WCHAR* msg, va_list argList) {       
    
	char*  currentTime = getCurrentTime(false);    
    logFile = fopen(logFullName, "a+");       
	
	fprintf(logFile, "%s [%s] - ", currentTime, level);
    // Write the WCHAR parameters
    vfwprintf(logFile, msg, argList);	
	fprintf(logFile, "\n"); 
	fclose(logFile);

    delete[] currentTime;	
}


void Log::reset(const char*  title) {
    
    const char *t = (title) ? title : "Funambol SDK C++ Log";

    char*  currentTime = getCurrentTime(true);
    memset(logFullName, 0, 512*sizeof(char));
    sprintf(logFullName, T("%s%s"), logPath, logName);
    logFile = fopen(logFullName, T("w+"));      
    fprintf(logFile, T("%s - %s\n"), t, currentTime);
    fclose(logFile);

    delete[] currentTime;    
}


/*
void Log::printMessage(const char*  level, const char*  msg) {           	
	
    char*  currentTime = getCurrentTime(false);
    logFile     = _wfopen(logPath, TEXT("a+"));       
    fwprintf(logFile, TEXT("%s [%s] - %s\n"), currentTime, level, msg); fflush(logFile);
	fclose(logFile);
    delete[] currentTime;
	
}
*/

